from typing import TYPE_CHECKING, Any, List, Optional, Set, Type, TypeVar, Union, cast

from pydantic import BaseModel
from pydantic.schema import schema

from pydantic_openapi_schema import v3_0_3, v3_1_0

if TYPE_CHECKING:
    from typing import Dict

REF_PREFIX = "#/components/schemas/"

T = TypeVar("T", bound=Union[v3_0_3.OpenAPI, v3_1_0.OpenAPI])


class OpenAPI303PydanticSchema(v3_0_3.Schema):
    """Special `Schema` class to indicate a reference from pydantic class"""

    schema_class: Type[BaseModel]
    """the class that is used for generate the schema"""


class OpenAPI310PydanticSchema(v3_1_0.Schema):
    """Special `Schema` class to indicate a reference from pydantic class"""

    schema_class: Type[BaseModel]
    """the class that is used for generate the schema"""


def construct_open_api_with_schema_class(
    open_api_schema: T,
    schema_classes: Optional[List[Type[BaseModel]]] = None,
    scan_for_pydantic_schema_reference: bool = True,
    by_alias: bool = True,
) -> T:
    """
    Construct a new OpenAPI object, with the use of pydantic classes to produce JSON schemas

    :param open_api_schema: the base `OpenAPI` object
    :param schema_classes: pydanitic classes that their schema will be used "#/components/schemas" values
    :param scan_for_pydantic_schema_reference: flag to indicate if scanning for `PydanticSchemaReference` class
                                               is needed for "#/components/schemas" value updates
    :param by_alias: construct schema by alias (default is True)
    :return: new OpenAPI object with "#/components/schemas" values updated.
             If there is no update in "#/components/schemas" values, the original `open_api` will be returned.
    """
    specs = v3_1_0 if isinstance(open_api_schema, v3_1_0.OpenAPI) else v3_0_3

    copied_schema = open_api_schema.copy(deep=True)

    if scan_for_pydantic_schema_reference:
        extracted_schema_classes = extract_pydantic_types_to_openapi_components(
            obj=copied_schema, ref_class=specs.Reference
        )
        schema_classes = list(
            {*schema_classes, *extracted_schema_classes} if schema_classes else extracted_schema_classes
        )

    if not schema_classes:
        return open_api_schema

    if not copied_schema.components:
        copied_schema.components = specs.Components(schemas={})
    elif not copied_schema.components.schemas:
        copied_schema.components.schemas = cast("Dict[str, Any]", {})

    schema_classes.sort(key=lambda x: x.__name__)
    schema_definitions = schema(schema_classes, by_alias=by_alias, ref_prefix=REF_PREFIX)["definitions"]
    copied_schema.components.schemas.update(  # type: ignore
        {key: specs.Schema.parse_obj(schema_dict) for key, schema_dict in schema_definitions.items()}
    )
    return cast("T", copied_schema)


def extract_pydantic_types_to_openapi_components(
    obj: Any, ref_class: Union[Type[v3_0_3.Reference], Type[v3_1_0.Reference]]
) -> Set[Type[BaseModel]]:
    """
    Recursively traverses the OpenAPI document, replacing any found Pydantic Models with $references to the schema's
    components section and returning the pydantic models themselves.

    :param obj:
    :param ref_class:
    :return set of pydantic schema classes:
    """
    pydantic_schemas: Set[Type[BaseModel]] = set()
    if isinstance(obj, BaseModel):
        fields = obj.__fields_set__
        for field in fields:
            child_obj = getattr(obj, field)
            if isinstance(child_obj, (OpenAPI303PydanticSchema, OpenAPI310PydanticSchema)):
                setattr(obj, field, ref_class(ref=REF_PREFIX + child_obj.schema_class.__name__))
                pydantic_schemas.add(child_obj.schema_class)
            else:
                pydantic_schemas.update(extract_pydantic_types_to_openapi_components(child_obj, ref_class=ref_class))
    elif isinstance(obj, list):
        for index, elem in enumerate(obj):
            if isinstance(elem, (OpenAPI303PydanticSchema, OpenAPI310PydanticSchema)):
                obj[index] = ref_class(ref=REF_PREFIX + elem.schema_class.__name__)
                pydantic_schemas.add(elem.schema_class)
            else:
                pydantic_schemas.update(extract_pydantic_types_to_openapi_components(elem, ref_class=ref_class))
    elif isinstance(obj, dict):
        for key, value in obj.items():
            if isinstance(value, (OpenAPI303PydanticSchema, OpenAPI310PydanticSchema)):
                obj[key] = ref_class(ref=REF_PREFIX + value.schema_class.__name__)
                pydantic_schemas.add(value.schema_class)
            else:
                pydantic_schemas.update(extract_pydantic_types_to_openapi_components(value, ref_class=ref_class))
    return pydantic_schemas



from typing import Dict, Union

from .reference import Reference
from .response import Response

Responses = Dict[str, Union[Response, Reference]]
"""
A container for the expected responses of an operation.
The container maps a HTTP response code to the expected response.

The documentation is not necessarily expected to cover all possible HTTP response codes
because they may not be known in advance.
However, documentation is expected to cover a successful operation response and any known errors.

The `default` MAY be used as a default response object for all HTTP codes
that are not covered individually by the specification.

The `Responses Object` MUST contain at least one response code, and it
SHOULD be the response for a successful operation call.

Fixed Fields

default: Optional[Union[Response, Reference]]

The documentation of responses other than the ones declared for specific HTTP response codes.
Use this field to cover undeclared responses.

Patterned Fields
{httpStatusCode]: Optional[Union[Response, Reference]]

Any `HTTP status code <https://spec.openapis.org/oas/v3.0.3#httpCodes>`__ can be used as the property name,
but only one property per code, to describe the expected response for that HTTP status code.
"""



from typing import Optional, Union

from pydantic import AnyUrl, BaseModel, EmailStr, Extra, validator


class Contact(BaseModel):
    """
    Contact information for the exposed API.
    """

    name: Optional[str] = None
    """
    The identifying name of the contact person/organization.
    """

    url: Optional[AnyUrl] = None
    """
    The URL pointing to the contact information. MUST be in the format of a URL.
    """

    email: Optional[Union[EmailStr, str]] = None
    """
    The email address of the contact person/organization. MUST be in the format of an email address.
    """

    @validator("email", pre=True)
    def validate_email(  # pylint: disable=no-self-argument
        cls,
        v: Union[EmailStr, str],
    ) -> EmailStr:
        """Validates that email is a valid email address

        Args:
            v (EmailStr|str): Holds the email string to be validated

        Raises:
            ValueError: Value is not a valid email address

        Returns:
            _type_: CompressionBackend
        """
        if isinstance(v, str):
            v = EmailStr(v)
        return v

    class Config:
        extra = Extra.ignore
        schema_extra = {
            "examples": [
                {"name": "API Support", "url": "http://www.example.com/support", "email": "support@example.com"}
            ]
        }



from pydantic import Extra, Field
from typing_extensions import Literal

from .parameter import Parameter


class Header(Parameter):
    """
    The Header Object follows the structure of the `Parameter Object <https://spec.openapis.org/oas/v3.0.3#parameterObject>`__ with the following changes:

    1. `name` MUST NOT be specified, it is given in the corresponding `headers` map.
    2. `in` MUST NOT be specified, it is implicitly in `header`.
    3. All traits that are affected by the location MUST be applicable to a location of `header`
       (for example, `style <https://spec.openapis.org/oas/v3.0.3#parameterStyle>`__).
    """

    name: Literal[""] = Field(default="", const=True)
    param_in: Literal["header"] = Field(default="header", const=True, alias="in")

    class Config:
        extra = Extra.ignore
        allow_population_by_field_name = True
        schema_extra = {
            "examples": [
                {"description": "The number of allowed requests in the current period", "schema": {"type": "integer"}}
            ]
        }



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydantic_openapi_schema',
 'pydantic_openapi_schema.utils',
 'pydantic_openapi_schema.v3_0_3',
 'pydantic_openapi_schema.v3_1_0']

package_data = \
{'': ['*']}

install_requires = \
['email-validator', 'pydantic']

setup_kwargs = {
    'name': 'pydantic-openapi-schema',
    'version': '1.0.2',
    'description': "OpenAPI Schema using pydantic. Forked for Starlite-API from 'openapi-schema-pydantic'.",
    'long_description': None,
    'author': "Na'aman Hirschfeld",
    'author_email': 'nhirschfeld@gmail.com',
    'maintainer': "Na'aman Hirschfeld",
    'maintainer_email': 'nhirschfeld@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)



from typing import List, Optional

from pydantic import BaseModel, Extra


class ServerVariable(BaseModel):
    """An object representing a Server Variable for server URL template substitution."""

    enum: Optional[List[str]] = None
    """
    An enumeration of string values to be used if the substitution options are from a limited set.
    The array SHOULD NOT be empty.
    """

    default: str
    """
    **REQUIRED**. The default value to use for substitution,
    which SHALL be sent if an alternate value is _not_ supplied.
    Note this behavior is different than the `Schema Object's <https://spec.openapis.org/oas/v3.1.0#schemaObject>`__ treatment of default values,
    because in those cases parameter values are optional.
    If the `enum <https://spec.openapis.org/oas/v3.1.0#serverVariableEnum>`__ is defined, the value MUST exist in the enum's values.
    """

    description: Optional[str] = None
    """
    An optional description for the server variable.
    `CommonMark syntax <https://spec.commonmark.org/>`__ MAY be used for rich text representation.
    """

    class Config:
        extra = Extra.ignore
